@@ -57,7 +57,9 @@ struct UICStruct {
 	gboolean getSms;
 	gulong sigId;
 	gboolean phoneOpened;
+	gboolean pollingPhone;
 	gboolean noConfig;
+	gint callDetected;
 };
 typedef struct UICStruct UIComponents;
 
@@ -108,6 +110,17 @@ static void  quit_event( UIComponents *uic, guint callback_action, GtkWidget *me
 					
 }
 
+/* callback to check availability of contactinfos service */
+gboolean pollPhone_cb(UIComponents *uic) {
+	
+	if (uic->phoneOpened && uic->pollingPhone) {
+		uic->pollingPhone = FALSE;
+		gn_sm_loop(1, state);
+		uic->pollingPhone = TRUE;
+	}
+	return TRUE;
+}
+
 /* callback to check availability of contactinfos service */
 gboolean checkAvailability_cb(UIComponents *uic) {
 	
@@ -214,16 +227,6 @@ void on_menu_about(GtkMenuItem * menuitem, UIComponents *uic) {
 void on_menu_transform(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
 	uic->transformPlus = !(uic->transformPlus);
 }
-/*
-void incoming_call(int lowlevel_id, char *number) {
-*/
-	/* sometimes the callback function gets called twice for the same call */
-/*	if (call_detected == lowlevel_id) return;
-	
-	printf("%s\n", number);
-	call_detected = lowlevel_id;
-	quit = 1;
-}
 
 gn_error hangup(int lowlevel_id) {
 	gn_call_info	callinfo;
@@ -250,7 +253,7 @@ gn_error answer(int lowlevel_id) {
 
 	return gn_sm_functions(GN_OP_AnswerCall, &data, state);
 }
-*/
+
 
 void notify_sms(gn_sms *message, struct gn_statemachine *state, UIComponents *uic) {
 	/* in this callback function you can't use those libgnokii functions that send a packet */
@@ -259,16 +262,45 @@ void notify_sms(gn_sms *message, struct gn_statemachine *state, UIComponents *ui
 }
 
 void notify_call(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, UIComponents *uic) {
+	gchar *descr;
+	GtkWidget *note;
+	gint resultCode;
+	
 	/* in this callback function you can't use those libgnokii functions that send a packet */
 	
+	if (call_status != GN_CALL_Incoming) {
+		return;
+	}
+		
+	if (uic->callDetected == call_info->call_id) {
+		return;
+	}
+
 	g_printf("\n===== ===== Incoming Call ===== =====\n");
 	
-	if (call_status != GN_CALL_Incoming) return;
+	uic->callDetected = call_info->call_id;
+	
+	descr = g_strdup_printf(_("Incoming Call\nnumber: %s\nname: %s"), call_info->number, call_info->name);
+	note = hildon_note_new_confirmation_add_buttons(NULL, descr, _("Answer"), GTK_RESPONSE_ACCEPT, _("Hang Up"), GTK_RESPONSE_REJECT, _("Ignore"), GTK_RESPONSE_CANCEL, NULL);
+	g_free(descr);
+	gtk_widget_show_all (note);
+
+	resultCode = gtk_dialog_run (GTK_DIALOG (note));
+	
+	switch (resultCode) {
+	case GTK_RESPONSE_ACCEPT:
+		answer(call_info->call_id);
+		break;
+	case GTK_RESPONSE_REJECT:
+		hangup(call_info->call_id);
+		break;
+	default:
+		break;
+	}
+	gtk_widget_destroy (note);
+
+	uic->callDetected = -1;
 	
-	hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_INFO, _("A call from %s"), call_info->number);
-	/*
-	incoming_call(call_info->call_id, call_info->number);
-	*/
 }
 
 gboolean getCalls(GtkWidget *topLevel, UIComponents *uic) {
@@ -296,15 +328,19 @@ gboolean getCalls(GtkWidget *topLevel, UIComponents *uic) {
 			return FALSE;
 		} else {
 			hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Waiting for a call ..."));
+			uic->pollingPhone = TRUE;
 		}
 	} else {
 		hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Stopped waiting for a call ..."));
 		gn_data_clear(&data);
 		data.call_notification = NULL;
 		gn_sm_functions(GN_OP_SetCallNotification, &data, state);
-		if (uic->phoneOpened && !uic->getSms) {
-			uic->phoneOpened = !(uic->phoneOpened);
-			gn_lib_phone_close(state);
+		if (!uic->getSms) {
+			uic->pollingPhone = FALSE;
+			if (uic->phoneOpened) {
+				uic->phoneOpened = !(uic->phoneOpened);
+				gn_lib_phone_close(state);
+			}
 		}
 	}
 	return TRUE;
@@ -343,15 +379,19 @@ gboolean getSms(GtkWidget *topLevel, UIComponents *uic) {
 			return FALSE;
 		} else {
 			hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Waiting for SMS ..."));
+			uic->pollingPhone = TRUE;
 		}
 	} else {
 		hildon_banner_show_informationf(topLevel, GTK_STOCK_DIALOG_INFO, _("Stopped waiting for SMS ..."));
 		gn_data_clear(&data);
 		data.on_sms = NULL;
 		gn_sm_functions(GN_OP_OnSMS, &data, state);
-		if (uic->phoneOpened && !uic->getCalls) {
-			uic->phoneOpened = !(uic->phoneOpened);
-			gn_lib_phone_close(state);
+		if (!uic->getCalls) {
+			uic->pollingPhone = FALSE;
+			if (uic->phoneOpened) {
+				uic->phoneOpened = !(uic->phoneOpened);
+				gn_lib_phone_close(state);
+			}
 		}
 	}
 	return TRUE;
@@ -1178,6 +1218,11 @@ int main( int argc, char *argv[] ) {
 	/* check availability and show or hide the Buttons */
 	g_idle_add ((GSourceFunc)checkAvailability_cb, uic);
 
+	uic->callDetected = -1;
+
+	/* check availability and show or hide the Buttons */
+	g_idle_add ((GSourceFunc)pollPhone_cb, uic);
+
 	if (!uic->noConfig) {
 		if (uic->getSms) {
 			getSms(GTK_WIDGET(appview), uic);
